###################################################
### chunk number 1: 
###################################################
options(prompt="R> ")
options(continue = "+  ")
options(width = 60)
options(useFancyQuotes = FALSE)
strOptions(strict.width = TRUE)
library(grid)
library(lattice)



###################################################
### chunk number 2: 
###################################################
chess <- read.table("chessmod.txt", sep = ":", quote = "",
                    col.names = c("player1", "player2",
                      "result", "moves", "year", "place",
                      "openingDetailed"))
chess$result <- factor(ifelse(chess$result == "1-1",
                              "draw",
                              ifelse((chess$result == "1-0" &
                                      chess$player1 == "La Bourdonnais") |
                                     (chess$result == "0-1" &
                                      chess$player2 == "La Bourdonnais"),
                                     "win",
                                     "loss")),
                       levels = c("win", "draw", "loss"))
chess$opening <- reorder(factor(gsub("^... |, .+$", "", 
                                     chess$openingDetailed)),
                         chess$moves, FUN = median)
chess$draw <- ifelse(chess$result == "draw", "draw", "result")

chess.tab <- xtabs( ~ moves + result, chess)
chess.tab.df <- as.data.frame(chess.tab)
chess.tab.df$nmoves <- as.numeric(as.character(chess.tab.df$moves))

chess.df <- subset(chess.tab.df, Freq > 0)
# Fiddle to force y-scale to include 0
chess.df <- rbind(chess.df, 
                  data.frame(moves = NA, result = "win",
                             Freq = 0, nmoves = 1000))



###################################################
### chunk number 3: chessplot eval=FALSE
###################################################
## xyplot(Freq ~ nmoves | result, data = chess.df, type = "h", 
##        layout = c(1, 3), xlim = c(0, 100))   
## 


###################################################
### chunk number 4: chess
###################################################
print(
# One tiny hidden fiddle to control y-axis labels
xyplot(Freq ~ nmoves | result, data = chess.df, type = "h", 
       layout = c(1, 3), xlim = c(0, 100),
       scales = list(y = list(at = seq(0, 6, 2))))   
)



###################################################
### chunk number 5: chesspiece
###################################################
library("grImport")
PostScriptTrace("chess_game_01.fromInkscape.eps")
chessPicture <- readPicture("chess_game_01.fromInkscape.eps.xml")
pawn <- chessPicture[205:206]
# grid.newpage()
grid.picture(pawn)



###################################################
### chunk number 6: chesspluspieceplot eval=FALSE
###################################################
## xyplot(Freq ~ nmoves | result, data = chess.df, type = "h", 
##        layout = c(1, 3), xlim = c(0, 100),
##        panel = function(...) { 
##            panel.xyplot(...)
##            grid.symbols(pawn, .05, .5, use.gc = FALSE,
##                         size = unit(.5, "npc"),
##                         gp = gpar(fill = switch(which.packet(),
##                                   "white", "grey", "black")))
##        })   
## 


###################################################
### chunk number 7: chesspluspiece
###################################################
print(
# Fiddle to force y-scale to include 0
xyplot(Freq ~ nmoves | result, data = chess.df, type = "h", 
       layout = c(1, 3), xlim = c(0, 100),
       scales = list(y = list(at = seq(0, 6, 2))),
       panel = function(...) { 
           panel.xyplot(...)
           grid.symbols(pawn, .05, .5, use.gc = FALSE,
                        size = unit(.5, "npc"),
                        gp = gpar(fill = switch(which.packet(),
                                  "white", "grey", "black")))
       })   
)



###################################################
### chunk number 8: 
###################################################



###################################################
### chunk number 9: petaltrace
###################################################
PostScriptTrace("petal.ps")



###################################################
### chunk number 10: petal
###################################################
petal <- readPicture("petal.ps.xml")
grid.newpage()
grid.picture(petal)



###################################################
### chunk number 11: 
###################################################
petalps <- readLines("petal.ps")
cat(petalps,
    sep = "\n")


###################################################
### chunk number 12: 
###################################################
cat(gsub("\t", "    ", 
         gsub("(source)=", "\n         \\1=",
              readLines("petal.ps.xml"))),
    sep = "\n")


###################################################
### chunk number 13:  eval=FALSE
###################################################
## PostScriptTrace("petal.ps")
## 
## 


###################################################
### chunk number 14: petaloutline
###################################################
pointify <- function(object, ...) {
    # Thin out the dots for a better diagram
    n <- length(object@x)
    subset <- c(1, seq(2, n, 3), n)
    gTree(children = gList(linesGrob(object@x[subset], object@y[subset], 
                                   default = "native",
                                   gp = gpar(col = "grey"), ...),
                         pointsGrob(object@x[subset], object@y[subset], 
                                    size = unit(2, "mm"), pch = 16, ...)))
}
grid.picture(petal, FUN = pointify)



###################################################
### chunk number 15: flowertrace
###################################################
PostScriptTrace("flower.ps")



###################################################
### chunk number 16: flower
###################################################
PSflower <- readPicture("flower.ps.xml")
grid.newpage()
grid.picture(PSflower)



###################################################
### chunk number 17: 
###################################################
cat(readLines("flower.ps"),
    sep = "\n")


###################################################
### chunk number 18: 
###################################################
cat(readLines("convert.ps"),
    sep = "\n")


###################################################
### chunk number 19:  eval=FALSE
###################################################
## PostScriptTrace("flower.ps")
## 
## 


###################################################
### chunk number 20: 
###################################################
flowerLines <- gsub("\t", "    ", 
                    gsub("(source)=", "\n         \\1=",
                         readLines("flower.ps.xml")))
flowerLines <- flowerLines[nchar(flowerLines) > 0]
moves <- grep("<move", flowerLines)
endpaths <- grep("</path", flowerLines)
startLine <- 1
for (i in 1:5) {
    cat(flowerLines[startLine:(moves[i])],
        "    ...",
        sep = "\n")
    startLine <- endpaths[i] - 1
}
cat(flowerLines[startLine:length(flowerLines)],
    sep = "\n")


###################################################
### chunk number 21: 
###################################################
flowerRGML <- xmlParse("flower.ps.xml") 
xpathApply(flowerRGML, "//path//rgb", 'xmlAttrs<-', 
           value = c(r = .3, g = .6, b = .8))
saveXML(flowerRGML, "blueflower.ps.xml")



###################################################
### chunk number 22: blueflower
###################################################
blueflower <- readPicture("blueflower.ps.xml")
grid.picture(blueflower)



###################################################
### chunk number 23: 
###################################################
petal <- readPicture("petal.ps.xml")



###################################################
### chunk number 24: 
###################################################
str(petal)



###################################################
### chunk number 25: 
###################################################
PSflower <- readPicture("flower.ps.xml")



###################################################
### chunk number 26: 
###################################################
str(PSflower@summary)



###################################################
### chunk number 27: 
###################################################
petals <- PSflower[2:3]



###################################################
### chunk number 28: 
###################################################
str(petals@summary)



###################################################
### chunk number 29: petals
###################################################
grid.picture(petals)



###################################################
### chunk number 30: 
###################################################
petal@summary@xscale
petal@summary@yscale



###################################################
### chunk number 31: 
###################################################
library("cluster")



###################################################
### chunk number 32: flowerplotcode eval=FALSE
###################################################
## xyplot(V8 ~ V7, data = flower, 
##        xlab = "Height", ylab = "Distance Apart",
##        panel = function(x, y, ...) { 
##            grid.symbols(PSflower, x, y, units = "native", 
##                         size = unit(5, "mm")) 
##        })
## 


###################################################
### chunk number 33: flowerplot
###################################################
trellis.device("pdf", file = "import-flowerplot.pdf",
               width = 6, height = 4, color = FALSE)
print(
xyplot(V8 ~ V7, data = flower, 
       xlab = "Height", ylab = "Distance Apart",
       panel = function(x, y, ...) { 
           grid.symbols(PSflower, x, y, units = "native", 
                        size = unit(5, "mm")) 
       })

)
dev.off()



###################################################
### chunk number 34: tiger eval=FALSE
###################################################
## PostScriptTrace("tiger.ps")
## tiger <- readPicture("tiger.ps.xml")
## grid.picture(tiger[-1])
## 


###################################################
### chunk number 35: 
###################################################
png("import-tiger.png", width=900, height=900)
PostScriptTrace("tiger.ps")
tiger <- readPicture("tiger.ps.xml")
grid.picture(tiger[-1])

dev.off()



###################################################
### chunk number 36: 
###################################################
cat(gsub("%.+$", "", petalps[grep("curveto", petalps)]))



###################################################
### chunk number 37: bezier
###################################################
# grid.newpage()
# Main diagram in 2x1 inch viewport (extra space is for labels)
pushViewport(viewport(width = unit(2, "inches")))
pushViewport(viewport(width = .9, height = .9,
                      xscale = c(-10, 10), yscale = c(10, 20)))
x <- c(-5, -10, 10, 5)
y <- c(10, 20, 20, 10)
grid.circle(x, y, default = "native", r = unit(1, "mm"), 
            gp = gpar(col = NA, fill = "grey"))
grid.segments(x[1], y[1], x[2], y[2],
              default = "native",
              gp = gpar(col = "grey"))
grid.segments(x[3], y[3], x[4], y[4],
              default = "native",
              gp = gpar(col = "grey"))
grid.text("(-5, 10)    ", x[1], y[1], default = "native", 
          just = c("right", "bottom"), gp = gpar(cex = .5))
grid.text("(-10, 20)   ", x[2], y[2], default = "native", 
          just = c("right", "top"), gp = gpar(cex = .5))
grid.text("   (10, 20)", x[3], y[3], default = "native", 
          just = c("left", "top"), gp = gpar(cex = .5))
grid.text("    (5, 10)", x[4], y[4], default = "native", 
          just = c("left", "bottom"), gp = gpar(cex = .5))
Ms <- 1/6*rbind(c(1, 4, 1, 0),
                c(-3, 0, 3, 0),
                c(3, -6, 3, 0),
                c(-1, 3, -3, 1))
Msinv <- solve(Ms)
# Bezier control matrix
Mb <- rbind(c(1, 0, 0, 0),
            c(-3, 3, 0, 0),
            c(3, -6, 3, 0),
            c(-1, 3, -3, 1))
# Get B-spline control points from Bezier control points by
# Msinv %*% Mb %*% bezier control points
xs <- Msinv %*% Mb %*% x
ys <- Msinv %*% Mb %*% y
grid.xspline(xs, ys,
             default = "native",
             shape = 1, repEnds = FALSE,
             gp = gpar(col = "black", lwd = 2))
popViewport(2)



###################################################
### chunk number 38: flatbezier
###################################################
PostScriptTrace("petal.ps", "petalrough.xml", setflat = 3)
petalrough <- readPicture("petalrough.xml")
# grid.newpage()
xf <- petalrough@paths[[1]]@x
xf <- xf[-1]
xf <- rev(xf)[-1]
yf <- petalrough@paths[[1]]@y
yf <- yf[-1]
yf <- rev(yf)[-1]
pushViewport(viewport(width = unit(2, "inches")))
pushViewport(dataViewport(x, y))
x <- c(-50, -100, 100, 50)
y <- c(100, 200, 200, 100)
grid.circle(x, y, default = "native", r = unit(1, "mm"), 
            gp = gpar(col = NA, fill = "grey"))
grid.segments(x[1], y[1], x[2], y[2],
              default = "native",
              gp = gpar(col = "grey"))
grid.segments(x[3], y[3], x[4], y[4],
              default = "native",
              gp = gpar(col = "grey"))
grid.text("(-5, 10)    ", x[1], y[1], default = "native", 
          just = c("right", "bottom"), gp = gpar(cex = .5))
grid.text("(-10, 20)   ", x[2], y[2], default = "native", 
          just = c("right", "top"), gp = gpar(cex = .5))
grid.text("   (10, 20)", x[3], y[3], default = "native", 
          just = c("left", "top"), gp = gpar(cex = .5))
grid.text("    (5, 10)", x[4], y[4], default = "native", 
          just = c("left", "bottom"), gp = gpar(cex = .5))
grid.lines(xf, yf, 
           default = "native")
grid.circle(xf, yf, r = unit(.5, "mm"), 
            default = "native",
            gp = gpar(fill = "black"))
popViewport(2)



###################################################
### chunk number 39: 
###################################################
hellops <- readLines("hello.ps")
cat(hellops,
    sep = "\n")


###################################################
### chunk number 40: hellotrace
###################################################
PostScriptTrace("hello.ps")



###################################################
### chunk number 41: hello
###################################################
hello <- readPicture("hello.ps.xml")
grid.picture(hello)



###################################################
### chunk number 42: hellofill
###################################################
grid.picture(hello, fillText = TRUE)



###################################################
### chunk number 43: hellotexttrace
###################################################
PostScriptTrace("hello.ps", 
                "hellotext.xml",
                charpath = FALSE)



###################################################
### chunk number 44: hellotext
###################################################
hellotext <- readPicture("hellotext.xml")
grid.picture(hellotext)



###################################################
### chunk number 45: floweroutline
###################################################
grid.picture(PSflower, 
             use.gc = FALSE, 
             gp = gpar(fill = NA, col = "black"))



###################################################
### chunk number 46: flowerplotcode2 eval=FALSE
###################################################
## xyplot(V8 ~ V7, data = flower, 
##        xlab = "Height", ylab = "Distance Apart",
##        panel=function(x, y, ...) { 
##            grid.symbols(PSflower, x, y, units = "native", 
##                         size = unit(5, "mm"),
##                         use.gc = FALSE, 
##                         gp = gpar(col = "white", fill = "black", lwd = .5)) 
##        })
## 


###################################################
### chunk number 47: flowerplot2
###################################################
trellis.device("pdf", file = "import-flowerplot2.pdf",
               width = 6, height = 4, color = FALSE)
print(
xyplot(V8 ~ V7, data = flower, 
       xlab = "Height", ylab = "Distance Apart",
       panel=function(x, y, ...) { 
           grid.symbols(PSflower, x, y, units = "native", 
                        size = unit(5, "mm"),
                        use.gc = FALSE, 
                        gp = gpar(col = "white", fill = "black", lwd = .5)) 
       })

)
dev.off()



###################################################
### chunk number 48: 
###################################################
petalrgml <- readLines("petal.ps.xml")
petalrgml <- petalrgml[nchar(petalrgml) > 0]
petalrgml <- gsub("\t", "    ", 
                  gsub("(source)=", "\n         \\1=",
                       petalrgml))
cat(petalrgml[1:(grep("<move", petalrgml) + 1)], 
    "    ...",
    petalrgml[(grep("</path", petalrgml) - 1):length(petalrgml)],
    sep = "\n")



###################################################
### chunk number 49: 
###################################################
hellotextrgml <- readLines("hellotext.xml")
hellotextrgml <- hellotextrgml[nchar(hellotextrgml) > 0]
hellotextrgml <- gsub("\t", "    ", 
                      gsub("source=", "\n         source=",
                           hellotextrgml))
cat(hellotextrgml, sep = "\n")



###################################################
### chunk number 50: 
###################################################
slotNames(petal)



###################################################
### chunk number 51: 
###################################################
str(petal@paths[[1]])



###################################################
### chunk number 52: 
###################################################
str(hellotext@paths[[1]])



###################################################
### chunk number 53: 
###################################################
str(petal@summary)



###################################################
### chunk number 54: 
###################################################
cat(readLines("blueshade.R"), sep = "\n")



###################################################
### chunk number 55: 
###################################################
source("blueshade.R")


###################################################
### chunk number 56: 
###################################################
cat(readLines("blueify.R"), sep = "\n")



###################################################
### chunk number 57: 
###################################################
source("blueify.R")


###################################################
### chunk number 58: bluetiger eval=FALSE
###################################################
## grid.picture(tiger[-1], 
##              FUN = blueify)
## 


###################################################
### chunk number 59: 
###################################################
png("import-bluetiger.png", width=900, height=900)
grid.picture(tiger[-1], 
             FUN = blueify)

dev.off()



###################################################
### chunk number 60: gnulogo
###################################################
PostScriptTrace("GNU.ps")
GNU <- readPicture("GNU.ps.xml")
grid.picture(GNU)



###################################################
### chunk number 61: gnulogozoom
###################################################
xscale <- GNU@summary@xscale
yscale <- GNU@summary@yscale
xd <- diff(range(xscale))
yd <- diff(range(yscale))
grid.picture(GNU, 
             xscale = xscale[1] + c(.2, .7)*xd,
             yscale = yscale[1] + c(.4, .8)*yd)



###################################################
### chunk number 62: gnulogopaths
###################################################
picturePaths(GNU, nr = 1, nc = 2, label = FALSE)



###################################################
### chunk number 63: brokengnupaths
###################################################
brokenGNU <- explodePaths(GNU)
picturePaths(brokenGNU, nr = 3, nc = 5, 
             label = FALSE, freeScales = TRUE)



###################################################
### chunk number 64: 
###################################################
GNUlogo <- function() {
    xscale <- GNU@summary@xscale
    yscale <- GNU@summary@yscale
    grid.picture(brokenGNU[2],
                 xscale = xscale, yscale = yscale)
    grid.picture(brokenGNU[11:14], use.gc = FALSE, 
                 gp = gpar(fill = "white"),
                 xscale = xscale, yscale = yscale)
    grid.picture(brokenGNU[3:10],  
                 xscale = xscale, yscale = yscale)
}



###################################################
### chunk number 65: fixedgnupaths
###################################################
GNUlogo()



###################################################
### chunk number 66: logobody eval=FALSE
###################################################
## barchart(~ cit, main = "Number of Citations per Year", xlab = "",
##          panel = function(...) {
##              GNUlogo()
##              grid.rect(gp = gpar(fill = rgb(1, 1, 1, .9)))
##              panel.barchart(...)
##          })
## 


###################################################
### chunk number 67: logo
###################################################
cit <- c("1998"=4, "1999"=15, "2000"=17, "2001"=39, 
         "2002"=119, "2003"=276, "2004"=523, 
         "2005"=945, "2006"=1475, "2007"=2015) 
trellis.device("pdf", file = "import-logo.pdf", height = 4, color = TRUE)
print(
barchart(~ cit, main = "Number of Citations per Year", xlab = "",
         panel = function(...) {
             GNUlogo()
             grid.rect(gp = gpar(fill = rgb(1, 1, 1, .9)))
             panel.barchart(...)
         })

)
dev.off()



###################################################
### chunk number 68: 
###################################################
PostScriptTrace("page27.ps")



###################################################
### chunk number 69: 
###################################################
page27 <- readPicture("page27.ps.xml")
survivalPlot <- page27[c(3:16, 18, 27)]



###################################################
### chunk number 70: survivalplot
###################################################
pushViewport(viewport(gp = gpar(lex = .2)))
grid.picture(survivalPlot)
popViewport()



###################################################
### chunk number 71: 
###################################################
zeroY <- survivalPlot@paths[[9]]@y[1]
zeroY



###################################################
### chunk number 72: 
###################################################
unitY <- (survivalPlot@paths[[14]]@y[1] - zeroY)/100
unitY



###################################################
### chunk number 73: 
###################################################
greenY <- (survivalPlot@paths[[15]]@y - zeroY)/unitY
head(round(unname(greenY), 1), n = 20)



###################################################
### chunk number 74: 
###################################################
library("survival")
sfit <- survfit(Surv(time, status) ~ trt, data = veteran)
originalGreenY <- sfit$surv[1:sfit$strata[1]]
head(round(originalGreenY*100, 1), n = 9)



